# R Graphics with ggplot2
# Fall 2017
# UVa StatLab
# Clay Ford


# Helpful RStudio commands ------------------------------------------------

# Description       Windows & Linux       Mac 
# ----------------------------------------------------------------
# Run current line  Ctrl+Enter            Command+Enter
# Previous plot 	  Ctrl+Alt+F11 	        Command+Option+F11
#                   Ctrl+Shift+PageUp
# Next plot 	      Ctrl+Alt+F12 	        Command+Option+F12
#                   Ctrl+Shift+PageDown

# Packages ----------------------------------------------------------------

# only submit these lines if you don't already have these packages installed. 
install.packages("ggplot2")
install.packages("gridExtra") # to fit multiple plots in a window
install.packages("dplyr") # for data manipulation
install.packages("hexbin") # for hexagonal binning

# We'll also use the scales package; when you install ggplot2, the scales
# package is also installed

library(ggplot2)
library(dplyr)

# Data --------------------------------------------------------------------

# Albemarle County real estate data.

# Downloaded from Office of Geographic Data Services, 19-Sept-2016
# http://www.albemarle.org/department.asp?department=gds&relpage=3914#Parcels

# Data for records with City identified as "CHARLOTTESVILLE", "EARLYSVILLE",
# "CROZET", "NORTH GARDEN","SCOTTSVILLE", "KESWICK"

# R script to download and prepare data:
# https://github.com/clayford/ggplot2/blob/master/albemarle_county_homes.R

# Read in data:
# homes <- read.csv("albemarle_real_estate.csv")
URL <- "http://people.virginia.edu/~jcf2d/workshops/ggplot2/albemarle_real_estate.csv"
homes <- read.csv(URL)

class(homes) # data.frame
str(homes) # view structure

# reorder the levels of Condition; this ensures plots using this variable will
# present the levels in a sensible order. 
levels(homes$Condition) # out of order
homes$Condition <- factor(homes$Condition, 
                          levels = c("Substandard", "Poor", "Fair", 
                                     "Average", "Good", "Excellent"))

levels(homes$Condition) # better


# One Variable ------------------------------------------------------------

# The histogram helps us see how a continous variable is distributed
ggplot(homes, aes(x=FinSqFt)) + geom_histogram()

# note the message regarding "stat_bin".
# Recall "stat_bin" is the stat for geom_histogram.

# Says the documentation: "You should always override this value, exploring
# multiple widths to find the best to illustrate the stories in your data."

# Try a new binwidth or bins
ggplot(homes, aes(x=FinSqFt)) + geom_histogram(binwidth=50)
ggplot(homes, aes(x=FinSqFt)) + geom_histogram(bins = 40)
# try some others! Beware, setting small binwidths can tax your computer.

# A frequency polygon is like a histogram, but counts are presented with lines
# instead of bars.
ggplot(homes, aes(x=FinSqFt)) + geom_freqpoly(binwidth=50)
ggplot(homes, aes(x=FinSqFt)) + geom_freqpoly(bins = 40)

# We can map City to color or fill to get a "stacked" histogram
ggplot(homes, aes(x=FinSqFt, fill = City)) + geom_histogram(bins = 40)
ggplot(homes, aes(x=FinSqFt, color = City)) + geom_freqpoly(bins = 40)


# To create a "true" histogram with density instead of count, (ie, the area of 
# the histogram is 1), set y=..density.. in the aes() function.  ..density.. is 
# avariable generated by stat_bin; see ?stat_bin

ggplot(homes, aes(x=FinSqFt, y=..density..)) + 
  geom_histogram(binwidth=250)
ggplot(homes, aes(x=FinSqFt, y=..density..)) + 
  geom_freqpoly(binwidth=250)


# The reason we might want a "True" histogram is to compare distributions with
# different counts.
ggplot(homes, aes(x=FinSqFt, y=..density.., color = City)) + 
  geom_freqpoly(binwidth=250) 


# Before we go further, important to note we can save a plot object and add to
# it:
p <- ggplot(homes, aes(x=FinSqFt))
p # nothing to see, no layers

# now add a geom layer
p + geom_freqpoly(binwidth=300)

# facet by condition:
p + geom_freqpoly(bins=75) + facet_wrap(~ City)



####################################
# YOUR TURN! 

# (1) Create a histogram or frequency polygon of TotalValue, the total value of
# the home. Try some different bandwidths, say bins = 50 or binwidth = 50000:


# (2) Repeat #1, but now facet by Condition



####################################

# Bar Plots help us visualize how discrete values are distributed:
ggplot(homes, aes(x=Condition)) + geom_bar()

# the coord_flip() function allows us to flip the coordinate axis; notice there
# is also a width argument we can use to adjust size of bars.
ggplot(homes, aes(x=Condition)) + geom_bar(width = 0.75) +
  coord_flip() 

# Let's map the Remodeled indicator to fill to get counts of both remodeled and
# original condition homes; The result is weird since since Remodeled is either
# 0 or 1
ggplot(homes, aes(x=Condition, fill=Remodeled)) + geom_bar()

# It helps to declare Remodeled a "factor";
ggplot(homes, aes(x=Condition, fill=factor(Remodeled))) + geom_bar()

# Note the "stacked" position. We can change that by setting position = "dodge"
# in geom_bar():
ggplot(homes, aes(x=Condition, fill=factor(Remodeled))) + 
  geom_bar(position = "dodge")

# How can we fix the legend title?
# Recall that is a by-product of the fill scale. So we need to use a scale
# function.
ggplot(homes, aes(x=Condition, fill=factor(Remodeled))) + 
  geom_bar(position = "dodge") +
  scale_fill_discrete(name="Remodeled", labels = c("Original","Remodeled"))


####################################
# YOUR TURN! 

# (1) Create a barplot of counts for City


# (2) Repeat #1, but now map the fill aesthetic to Condition



####################################



# Two variables -----------------------------------------------------------

# The scatterplot allows you to visualize the relationship between two
# continuous variables.

# plot FinSqFt vs TotalValue:
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point()

# Lots of overplotting!
# What can we do about that? One solution is the alpha aesthetic. "alpha=1/10"
# means 10 points overplotted adds to a solid color
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point(alpha=1/10)

# Can also try smaller points using the shape aesthetic
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point(shape=".")

# Another approach is to use facets: break the data into subsets 
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point() +
  facet_wrap(~ City)

# We can let the scales vary in the facets by setting scales = "free"
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point() +
  facet_wrap(~ City, scales = "free")

# and we can combine our tactics
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point(shape = ".") +
  facet_wrap(~ City, scales = "free")


# yet another strategy is to bin the points and count the number in each using
# geom_hex()
# install.packages("hexbin")
library(hexbin)

ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_hex()

ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_hex(bins = 50)


# We can also map the color of points to a variable in our data.
ggplot(homes, aes(x=FinSqFt, y=TotalValue, color=factor(Remodeled))) + 
  geom_point() +
  facet_wrap(~ City)

# Again we get a legend with a title that we need to fix. One way to fix it is 
# to update the variable in the data frame. In this case we could change it to a
# factor.
homes$Remodeled <- factor(homes$Remodeled, labels = c("Original","Remodeled"))

# Now we can create our plot without using factor()
ggplot(homes, aes(x=FinSqFt, y=TotalValue, color=Remodeled)) + 
  geom_point() +
  facet_wrap(~ City)


#################################### 
# YOUR TURN! Plot FinSqFt vs. YearBuilt. Put FinSqFt on the y axis. 
# Facet by City and map color to Condition.



####################################

# Let's look again at TotalValue vs FinSqFt:
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point(alpha=1/6) +
  facet_wrap(~ City)

# The y-axis scale would look better formatted as dollar amounts. The scales
# package can help with this. It has functions designed for this type of
# situation. When you install ggplot2, scales is installed as well.

# Three functions that are very handy: percent, comma, dollar;
# typically used in a scale function with the labels argument.

library(scales)
p2 <- ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point(alpha=1/6) +
  facet_wrap(~ City) + 
  scale_y_continuous(labels=dollar) + 
  scale_x_continuous(labels=comma)
p2

# We can add a smooth line through our scatterplots with geom_smooth()
p2 + geom_smooth()

# turn off the confidence intervals by setting se=FALSE
p2 + geom_smooth(se=FALSE)

# We can fit a straight linear regression line setting method="lm"
p2 + geom_smooth(method="lm")

# We can add both smooth and straight lines
p2 + 
  geom_smooth(se=FALSE, color="red") +
  geom_smooth(method="lm", se=FALSE)

# But notice ggplot2 does not create a legend for us identifying the lines!
# That's because the lines are created from statistical transformations of the
# data. There's nothing to map the color of the lines to in the data.
  
# However we can force a legend by defining a color aesthetic in each geom:
p2 + 
  geom_smooth(aes(color = "smooth"), se=FALSE) +
  geom_smooth(aes(color = "linear"), method="lm", se=FALSE)

# And then we can customize the legend if we like
p2 + 
  geom_smooth(aes(color = "smooth"), se=FALSE) +
  geom_smooth(aes(color = "linear"), method="lm", se=FALSE) +
  scale_color_manual("Fitted Lines", values = c("blue","red"))

# Let's zoom in on the plots to get a closer look at what's going on in the 
# lower left corner. We can zoom in using coord_cartesian(). Below we set the 
# x-axis limits between 0 and 5000, and the y-axis limits between 0 and
# 1,000,000.
p2 + 
  geom_smooth(aes(color = "smooth"), se=FALSE) +
  geom_smooth(aes(color = "linear"), method="lm", se=FALSE)+
  scale_color_manual("Fitted Lines", values = c("blue","red")) +
  coord_cartesian(xlim = c(0, 5000), ylim = c(1,1e6))

# To add a title we can use ggtitle()
p2 + 
  geom_smooth(aes(color = "smooth"), se=FALSE) +
  geom_smooth(aes(color = "linear"), method="lm", se=FALSE)+
  scale_color_manual("Fitted Lines", values = c("blue","red")) +
  coord_cartesian(xlim = c(0, 5000), ylim = c(1,1e6)) +
  ggtitle("Total Values vs Finished Sq Ft by City") 

# Notice the title is left-aligned by default. Here's how we can center it using
# the theme() function.
p2 + 
  geom_smooth(aes(color = "smooth"), se=FALSE) +
  geom_smooth(aes(color = "linear"), method="lm", se=FALSE)+
  scale_color_manual("Fitted Lines", values = c("blue","red")) +
  coord_cartesian(xlim = c(0, 5000), ylim = c(1,1e6)) +
  ggtitle("Total Values vs Finished Sq Ft by City") +
  theme(plot.title = element_text(hjust = 0.5))

# Log Transformations in scatter plots

# Dollar amounts are often log transformed. It helps "squeeze together" the 
# large values and "spread out" the small values.

# before log transformation:
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + geom_point()

# Maybe try log base 10 transformation directly:
ggplot(homes, aes(x=log10(FinSqFt), y=log10(TotalValue))) + geom_point(alpha = 1/5)

# The scale of the axes is on the log10 scale because the data has been
# transformed.

# We can use scale functions to both transform the data and map the scale of the
# axes to the original data.
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point(alpha=1/5) +
  scale_x_log10(labels=comma) + 
  scale_y_log10(labels=dollar)

# We can see the non-linear nature of the scales by manually defining the breaks
ggplot(homes, aes(x=FinSqFt, y=TotalValue)) + 
  geom_point(alpha=1/5) +
  scale_x_log10(labels=comma, breaks = seq(0, 1e4, 2e3)) +  # 0 to 10,000 in steps of 2,000
  scale_y_log10(labels=dollar, breaks = seq(0, 10e6, 2e6))  # 0 to 10,000,000 in steps of 2,000,000

# Boxplots

# Boxplots are good for visualizing a continous variable conditional on a
# discrete variable. Let's look at TotalValue by number of FullBaths:

ggplot(homes, aes(x=FullBath, y=TotalValue)) + geom_boxplot()

# oops! FullBath is not discrete. Can fix by setting group = FullBath
ggplot(homes, aes(x = FullBath, y = TotalValue, group = FullBath)) + 
  geom_boxplot()

# and so does this (convert FullBath to factor):
ggplot(homes, aes(x=factor(FullBath), y=TotalValue)) + 
  geom_boxplot() + 
  scale_y_continuous(labels=dollar)

# The expensive homes have rendered this boxplot practically useless. We could 
# "subset" the data by changing the y-axis limits using the limits argument in
# scale_y_continuous; notice the warning message
ggplot(homes, aes(x=factor(FullBath), y=TotalValue)) + 
  geom_boxplot() + 
  scale_y_continuous(labels=dollar, limits = c(1,1e6))

# Another option might be to create a stripchart, which is basically a 
# scatterplot in one dimension. We can do that with geom_jitter() which is 
# basically geom_point() but with random noise added to the coordinates. Below
# we jitter left and right, but not up and down.
ggplot(homes, aes(x=factor(FullBath), y=TotalValue)) + 
  geom_jitter(width = 0.3, height = 0, alpha = 1/5) + 
  scale_y_continuous(labels=dollar, limits = c(1,1e6))

# Same plot just for 1 to 5 full bathrooms
ggplot(homes, aes(x=factor(FullBath), y=TotalValue)) + 
  geom_jitter(width = 0.4, height = 0, alpha = 1/5) + 
  scale_y_continuous(labels=dollar, limits = c(1,1e6)) +
  scale_x_discrete(breaks = 1:5, limits = 1:5) +
  labs(x = "Number of Full Bathrooms")


# similar to boxplots: violin plots. A violin plot is a mirrored density plot
# displayed in the same way as a boxplot.
ggplot(homes, aes(y = TotalValue, x = factor(FullBath))) + 
  geom_violin()

# Again, same plot just for 1 to 5 full bathrooms 
ggplot(homes, aes(y = TotalValue, x = factor(FullBath))) +
  geom_violin() +
  scale_y_continuous(labels=dollar, limits = c(1,1e6)) +
  scale_x_discrete(breaks = 1:5, limits = 1:5) +
  labs(x = "Number of Full Bathrooms")



####################################
# YOUR TURN! Make a boxplot (or violin plot) of FinSqFt by HalfBath. 



####################################

# Plotting two "discrete" integer variables:

# Bedroom vs. FullBath
ggplot(homes, aes(x=Bedroom,y=FullBath)) + geom_point()

# geom_jitter() can help with this
ggplot(homes, aes(x=Bedroom,y=FullBath)) + geom_jitter()

# scales could be better; minor_breaks=NULL turns off the minor grid lines 
ggplot(homes, aes(x=Bedroom,y=FullBath)) + geom_jitter(alpha=1/5) +
  scale_x_continuous(breaks=0:16, minor_breaks=NULL) +
  scale_y_continuous(breaks=0:14, minor_breaks=NULL)


# Line Graphs

# line graphs are nice for connecting dots and showing a trend over time.

# plot number of houses built per year; 
# need to count up number of homes by YearBuilt
years <- homes %>% 
  group_by(YearBuilt) %>% 
  tally()
years


# now use geom_line()
ggplot(years, aes(x=YearBuilt, y=n)) + 
  geom_line()

# Let's change the x-axis to show years in increments of 50
ggplot(years, aes(x=YearBuilt, y=n)) + 
  geom_line() +
  scale_x_continuous(breaks=seq(1700,2000,50)) +
  labs(x="Year",y="Number of Homes")

# what year was the spike?
filter(years, n > 800)

# we can add the year to the plot using annotate()
ggplot(years, aes(x=YearBuilt, y=n)) + 
  geom_line() +
  scale_x_continuous(breaks=seq(1700,2000,50)) +
  labs(x="Year",y="Number of Homes") +
  annotate("text", x = 1985, y = 900, label = "1973") 



# Plotting with two data frames -------------------------------------------


# ggplot allows us to use multiple data frames when creating a plot. For 
# example, consider the following plot TotalValue vs YearBuilt for the years 
# 1950 - 2016 for homes valued at $1,000,000 or less. Notice we are zoomed in.

ggplot(homes, aes(x=YearBuilt, y=TotalValue)) + 
  geom_point(position=position_jitter(w=0.2,h=0), shape=".") +
  coord_cartesian(xlim=c(1950,2016), ylim=c(0,1e6))

# Let's say we wanted to plot on top of that graph the mean and median 
# TotalValue by year. First we need to calculate the mean and median by year.
# Here's one way to do it using dplyr:

homeValues <- homes %>% 
  group_by(YearBuilt) %>% 
  summarize(mean_TotalValue = mean(TotalValue),
            median_TotalValue = median(TotalValue))
homeValues

# Each geom_xxx() function allows you to supply them with a separate data frame 
# and mappings. Below notice the two geom_line() functions each reference the 
# homeValues data frame and provide a different mapping to the y aesthetic. Also
# notice we have swap the order of data and aesthesics. 

ggplot(homes, aes(x=YearBuilt, y=TotalValue)) + 
  geom_point(position=position_jitter(w=0.2,h=0), shape=".") +
  geom_line(aes(y = mean_TotalValue, color = "mean"), homeValues) +
  geom_line(aes(y = median_TotalValue, color = "median"), homeValues) +
  coord_cartesian(xlim=c(1950,2016), ylim=c(0,1e6)) +
  scale_color_manual("Summary", values = c("blue", "red"))

# Another example

# Boxplot of TotalValues by City for homes $1,000,000 or less
ggplot(filter(homes, TotalValue <= 1e6), 
       aes(x = City, y = TotalValue)) +
  geom_boxplot()

# Let's say we wanted to label the medians with the actual amount.

# First we calculate it. Notice we create two versions, one numeric and one 
# formatted in dollar amounts using the dollar function from the scales package.
homeMedians <- homes %>% 
  group_by(City) %>% 
  filter(TotalValue <= 1e6) %>% 
  summarize(medianValue = median(TotalValue)) %>% 
  mutate(medianValueD = dollar(medianValue))

homeMedians

# Now we use the geom_text() function to add the amount to the boxplot. Notice 
# we use alpha = 1/4 to lighten the boxplot, that the label aesthetic says which
# data will be mapped to the text label, and that we add 25,000 to the 
# positioning so the label is above the median line. (that took some trial and
# error)
ggplot(filter(homes, TotalValue <= 1e6), 
       aes(x = City, y = TotalValue)) +
  geom_boxplot(alpha = 1/4) +
  geom_text(aes(label = medianValueD, y = medianValue + 25000), homeMedians) 



# Multiple plots in one window --------------------------------------------


# In base R, we usually use par(mfrow=c(i,j)), like so:
par(mfrow=c(1,2))
hist(homes$FinSqFt)
plot(FinSqFt ~ YearBuilt, data=homes)
par(mfrow=c(1,1))

# We cannot use this approach for ggplot2. The easiest solution is to use the 
# grid.arrange() function in the gridExtra package. To use it, you have to save
# your plots and then call them using grid.arrange().

p1 <- ggplot(homes, aes(x=FinSqFt)) + geom_histogram() 
p2 <- ggplot(homes, aes(x=YearBuilt, y=FinSqFt)) + geom_point()
library(gridExtra)
grid.arrange(p1, p2, nrow=1) # can also use ncol


# Themes

# Using built-in themes such as theme_bw()
# Before
ggplot(homes, aes(x=FinSqFt)) + geom_histogram(bins = 40)
# After
ggplot(homes, aes(x=FinSqFt)) + geom_histogram(bins = 40) + 
  theme_bw()
# another: theme_light
ggplot(homes, aes(x=FinSqFt)) + geom_histogram(bins = 40) + 
  theme_light()

# To permanently change the theme, use theme_set:
prevTheme <- theme_set(theme_bw())

ggplot(homes, aes(x=FinSqFt)) + geom_histogram(bins = 40)

ggplot(homes, aes(x=City, y = FinSqFt)) + geom_boxplot()



# To restore
theme_set(prevTheme)
# verify
ggplot(homes, aes(x=City, y = FinSqFt)) + geom_boxplot()

# see package ggthemes for more themes!
# install.packages("ggthemes")
library(ggthemes)

# Wall Street Journal theme
ggplot(homes, aes(x=City, y = FinSqFt)) + 
  geom_boxplot() + 
  theme_wsj()

# google docs theme
ggplot(homes, aes(x=City, y = FinSqFt)) + 
  geom_boxplot() + 
  theme_gdocs()

# fivethirtyeight.com theme
ggplot(homes, aes(x=City, y = FinSqFt)) + 
  geom_boxplot() + 
  theme_fivethirtyeight()

# Tufte's theme (based on The Visual Display of Quantitative Information)
ggplot(homes, aes(x=City, y = FinSqFt)) + 
  geom_boxplot() + 
  theme_tufte()

# base R theme
ggplot(homes, aes(x=City, y = FinSqFt)) + 
  geom_boxplot() + 
  theme_base()



# Time-Permitting Miscellaneous Topics ------------------------------------


# adding uncertainty to graphs 

# Example: add means and CIs to strip chart 
# Data: chickwts (An experiment was conducted to measure and compare the
# effectiveness of various feed supplements on the growth rate of chickens.)
str(chickwts)

sc <- ggplot(chickwts, aes(x=feed, y=weight)) + 
  geom_jitter(width = 0.1, height = 0)
sc

# now to calculate means and standard errors
chick2 <- chickwts %>% 
  group_by(feed) %>% 
  summarise(fMean = mean(weight),
            fSE = sd(weight)/sqrt(length(weight)))

# now add mean and error bars
sc + geom_point(data=chick2, aes(x=feed, y=fMean), color="red", size=3) +
  geom_errorbar(data=chick2, aes(x=feed, y=fMean,
                                 ymin=fMean - 2*fSE, 
                                 ymax=fMean + 2*fSE), 
                width=0.1, color="red") +
  labs(title="Mean Weight by Feed Type with 2*SE error bars")

# another way using stat_summary; fun.data="mean_cl_normal" actually calls the 
# function smean.cl.normal() from the Hmisc package. It uses the t distribution
# to determine the multiplier of the standard error.
ggplot(chickwts, aes(x=feed, y=weight)) + 
  geom_point(position = position_jitter(w = 0.1, h = 0)) +
  stat_summary(fun.data = "mean_cl_normal", color="red", geom="errorbar", width=0.1) +
  stat_summary(fun.y = "mean", geom="point", color="red", size=3) +
  labs(title="Mean Weight by Feed Type with 95% error bars")



# single line graph of means at each time point with SE bars

# Indometh data (comes with R)
# Six subjects were given an intravenous injection of indomethacin at 11 times,
# and each time plasma concentrations of indometacin was measured.

# Here we make a line plot for each subject:
ggplot(Indometh, aes(x=time,y=conc, group=Subject)) +
  geom_line()

# Let's say we wanted to create a single line graph of means at each time point
# with SE bars.

# first calculate means and SEs
Indo2 <- Indometh %>% 
  group_by(time) %>% 
  summarise(tMean = mean(conc),
            tSE = sd(conc)/sqrt(length(conc)))

# now ready to create plot
ggplot(Indo2, aes(x=time,y=tMean)) +
  geom_line() +
  geom_errorbar(aes(ymin=tMean-2*tSE, ymax=tMean+2*tSE), width=0.1)

# or again we can use stat_summary()
ggplot(Indometh, aes(x=time,y=conc)) +
  stat_summary(fun.data="mean_cl_normal", geom="errorbar", width=0.1) +
  stat_summary(fun.y = "mean", geom = "line")


# maps
# ggplot can plot maps

# The following data are occurences of cougars in the US. It was downloaded from
# Biodiversity Information Serving Our Nation (BISON),
# http://bison.usgs.ornl.gov.

URL2 <- "http://people.virginia.edu/~jcf2d/workshops/ggplot2/bison-Cougar-20150520-172801.csv"
cougar <- read.csv(URL2)

# install.packages("maps")
# install.packages("mapproj")

library(maps)
library(mapproj) # for the "polyconic" option

# Let's plot a map that shows occurences of courgars in the US and the type of occurence.

# first create a data frame of map data; "state" is the name of a map provided
# by the maps package.
states <- map_data("state")

# rename providedState to "region" and make lower case for purposes of merging
cougar$region <- tolower(cougar$providedState)

# merge states and courgar data frames by region, and also drop records if
# they're missing data for decimalLongitude.
cougmap <- filter(cougar, !is.na(decimalLongitude)) %>% 
  inner_join(states, by = "region")

# now we're ready to plot! Map longitude and latitude to x and y aesthetics;
# The borders function draws the state borders;
# the coord_map("polyconic") function makes the map look curvy, like a globe;
# Give it a second...
ggplot(cougmap, aes(x=long, y=lat)) +
  borders("state") +
  geom_point(aes(x = decimalLongitude, y=decimalLatitude, color=basisOfRecord)) +
  coord_map("polyconic")

# saving graphs as images

# use ggsave(). It saves the last plot according to your file extension.
ggsave("cougars.jpg", width=10, height=5)



# END SCRIPT